/*
 * Ben NanoNote and EM027AS012 E-Paper Display Adapter Board communication.
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * SPI functions derived from those in lib/atben.c by Werner Almesberger:
 *
 * Copyright (C) 2010-2011 Werner Almesberger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Driver-related code originally provided under the following terms:
 *
 * Copyright 2013 Pervasive Displays, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

#include <ubb/ubb.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <stdio.h>
#include "bsp.h"

void EPD_Pin_init()
{
    OUT(UBB_DAT2);
    OUT(UBB_DAT3);
    OUT(UBB_CMD);
    OUT(UBB_CLK);
    OUT(UBB_DAT0);
    OUT(UBB_DAT1);

    CLR(UBB_DAT2);
    CLR(UBB_DAT3);
    CLR(UBB_CMD);
    CLR(UBB_CLK);
    CLR(UBB_DAT0);
    CLR(UBB_DAT1);
}

void EPD_Pin_EPD_CS(EPD_pinstate pin)
{
    if (pin == HIGH)
        SET(EPD_SSEL);
    else
        CLR(EPD_SSEL);
}

void EPD_Pin_RESET(EPD_pinstate pin)
{
    if (pin == HIGH)
        SET(EPD_RESET);
    else
        CLR(EPD_RESET);
    printf("RESET = %d\n", pin);
}

void EPD_Pin_PANEL_ON(EPD_pinstate pin)
{
    if (pin == HIGH)
        ubb_power(1);
    else
        ubb_power(0);
    printf("PANEL = %d\n", pin);
}

void EPD_Pin_DISCHARGE(EPD_pinstate pin)
{
    if (pin == HIGH)
        SET(EPD_DIS);
    else
        CLR(EPD_DIS);
    printf("DIS = %d\n", pin);
}

void EPD_Pin_BORDER(EPD_pinstate pin)
{
}

void EPD_Pin_PWM(EPD_pinstate pin)
{
    if (pin == HIGH)
        SET(EPD_PWM);
    else
        CLR(EPD_PWM);
}

EPD_pinstate epd_get_busy()
{
    bsp_delayUs(1);
    return LOW;
}

void bsp_spiWrite(uint8_t *buffer, uint16_t length)
{
    uint8_t mask, *ptr, *limit = buffer + length;

    for (ptr = buffer; ptr < limit; ptr++)
    {
        for (mask = 0x80; mask; mask >>= 1)
        {
            if (*(ptr) & mask)
            {
                SET(EPD_MOSI);
            }
            else
            {
                CLR(EPD_MOSI);
            }

            SET(EPD_SCK);
            CLR(EPD_SCK);
        }
    }
}

void bsp_delayMs(uint32_t ms)
{
    bsp_delayUs(ms * 1000);
}

void bsp_delayUs(uint32_t us)
{
    /* uint32_t now = bsp_getUsTicks(); */

    /* while (bsp_getUsTicks() - now < us); */

    uint32_t count = us * 24;
    while (count--);
}

uint32_t bsp_getMsTicks()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec * 1000 + tv.tv_usec / 1000;
}

uint32_t bsp_getUsTicks()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec * 1000000 + tv.tv_usec;
}

int16_t bsp_getTemp()
{
    return 20;
}
